/*
	Console Dictionary.cp
	
	Run the CDictionary related classes through their paces.
	
	by James Jennings
	
*/

#ifdef _MSC_VER
	// #include for the Microsoft Visual C++ environment
	#include "stdafx.h"
#endif

#include <assert.h>
#include <iostream>
#include <fstream>
#include "CLOD.h"
#include "CLODHistory.h"
#include "CDictionary.h"
#include "COStreamFormatter.h"
#include "CHTMLFormatter.h"
#include "CCGIFormatter.h"
#include "CPrimSet.h"
#include "CWordMaker.h"
#include "ComplexMaker.h"
#include "CEntryParser.h"
#include "CFullParse.h"


#include <time.h>	// for timing/profiling operations
#include <Events.h>	// for timing/profiling operations

#ifdef __MWERKS__
/* Change Codewarrior's save-on-quit behavior. */
#include <SIOUX.h>
#endif

using namespace std;  //introduces namespace std

#if 0
	// use these to get the files from a special directory (for testing only)
const char logdataname[]  = "up:Utilities:Loglan Utilities:LOD 1/10/01:Dict.log";
const char logindexname[] = "up:Utilities:Loglan Utilities:LOD 1/10/01:index.log";
const char engdataname[]  = "up:Utilities:Loglan Utilities:LOD 1/10/01:dict.eng";
const char engindexname[] = "up:Utilities:Loglan Utilities:LOD 1/10/01:index.eng";
const char primsetname[]  = "prim.set";
const char htmlfilename[] = "test.html";
const char CGIFormFileName[]= "CGIForm.html";
#else
	// use these to get the files from the "home" directory
const char logdataname[]  = "Dict.log";
const char logindexname[] = "index.log";
const char engdataname[]  = "dict.eng";
const char engindexname[] = "index.eng";
const char primsetname[]  = "prim.set";
const char htmlfilename[] = "test.html";
const char CGIFormFileName[]= "CGIForm.html";
#endif

void GetDefinition( CLOD &theDict, CFormatter &formatter);
void PrintParse( CLOD &theDict );
void EndianTest();
void CharacterScan(istream &is);

void ParseDictionary(CDictionary &dict);
void CompareWordMakers(CLEDictionary &dict);

int main()
{	
#ifdef __MWERKS__
	/* Change Codewarrior's save-on-quit behavior. */
	SIOUXSettings.autocloseonquit = true;
	SIOUXSettings.asktosaveonclose = false; 
#endif
	cout << "The Loglan Online Dictionary" << endl;
	cout << "============================" << endl << endl;
	
	ifstream logdata(  logdataname,  ios::in | ios::binary );
	ifstream logindex( logindexname, ios::in | ios::binary );
	ifstream engdata(  engdataname,  ios::in | ios::binary );
	ifstream engindex( engindexname, ios::in | ios::binary );
	ifstream primset(  primsetname,  ios::in | ios::binary );
	ifstream cgiform(  CGIFormFileName );
	
	try {
		
		if (!(logdata.is_open()) || !(logindex.is_open())) {
			cerr << "*** Couldn't open Loglan-English files. ***";
			exit(-1);
		}
		if (!(engdata.is_open()) || !(engindex.is_open())) {
			cerr << "*** Couldn't open English-Loglan files. ***";
			exit(-1);
		}
		
		if (primset.is_open()) {
			CPrimSet::Initialize(primset);
			primset.close();
		} else {
			cerr << "*** Couldn't open prim.set file. ***";
			cerr << "*** Word making functions are not available. ***";
		}
		
		string theCGIForm;
		if (cgiform.is_open()) {
			while(cgiform) {
				char s[100];
				cgiform.getline(s, 100);
				theCGIForm += s;
				theCGIForm += '\n';
			}
			cgiform.close();
		} else {
			cerr << "*** Couldn't open CGIForm.html file. ***";
			cerr << "*** CGI output is limited. ***";
		}
		
//		CharacterScan(logdata);	// count eight bit characters
//		return 0;
		
		CLOD theDict(logdata, logindex, engdata, engindex);
		CLODHistory theHistory;
		
#if 0	// exercising the parser and the data files
		CompareWordMakers(theDict.GetLE());
		ParseDictionary(theDict.GetLE());
		ParseDictionary(theDict.GetEL());
		exit(0);
#endif
		// for debugging
	//	theDict.GetLE().FindEntry("matma");
	//	theDict.GetEL().FindEntry("Loglan");
		// end debugging
		
		COStreamFormatter formatter;
		
		ofstream htmlfile;
		CHTMLFormatter html(htmlfile);
		
		bool looping = true;
		
		while (looping) {
			// display a summary of the choices
			cout << endl;
			if (theDict.WhichDictionary() == LE)
				cout << "Using the Loglan-English dictionary." << endl;
			else
				cout << "Using the English-Loglan dictionary." << endl;
	// Experimental CountEntries() test. Takes about 5 seconds on PM 8500/120.
	// Expected result: LE has 9,334, EL has 13,492
	// Actual result: LE 4533 in ~5 seconds, EL 6368 in ~5 seconds.
/*			float tc = ::TickCount();
			cout << " (LE: " << theDict.GetLE().CountEntries() << " entries counted in ";
			cout << (::TickCount()-tc)/60 << " seconds.)" << endl;
			tc = ::TickCount();
			cout << " (EL: " << theDict.GetEL().CountEntries() << " entries counted in ";
			cout << (::TickCount()-tc)/60 << " seconds.)" << endl;
*/	// end experiment
			cout << 
				"L)-E  E)-L  D)isplay  F)ind  N)ext  P)rev  M)ake H)istory  W)eb  Q)uit  ?) : "
				<< flush;
			
			// wait for input
			char what = ' ';
			cin >> what;
			
			cin.clear();
			cin.ignore(INT_MAX,'\n');
			
			// act on it
			switch(what) {
			case '?':	// help
				cout << endl;
				cout << "L) Use the Loglan-English dictionary." << endl;
				cout << "E) Use the English-Loglan dictionary." << endl;
				cout << "D) Display the current word." << endl;
				cout << "F) Find a word." << endl;
				cout << "N) Go to the next word." << endl;
				cout << "P) Go to the previous word." << endl;
				cout << "M) Make a complex word." << endl;
				cout << "H) Display history." << endl;
				cout << "W) Save the current word as an HTML file." << endl;
				cout << "Q) Quit the program." << endl;
				cout << "?) This list." << endl << endl;
				break;
				
			case 'l':	// switch to Loglan-English dictionary
			case 'L':
				theDict.ChooseDictionary(LE);
				break;
				
			case 'e':	// switch to English-Loglan dictionary
			case 'E':
				theDict.ChooseDictionary(EL);
				break;
				
			case 'd':	// display
			case 'D':
				GetDefinition(theDict, formatter);
				break;
				
			case 'n':	// next word
			case 'N':
				if ( ! theDict.CanDoNext() )
					cout << "*** Already at last word. ***" << endl;
				else
					theDict.NextEntry();
				GetDefinition(theDict, formatter);
				break;
				
			case 'p':	// previous word
			case 'P':
				if ( ! theDict.CanDoPrevious() )
					cout << "*** Already at first word. ***" << endl;
				else
					theDict.PreviousEntry();
				GetDefinition(theDict, formatter);
				break;
				
			case '#':
			{
				CCountedDictionary &cd = dynamic_cast<CCountedDictionary&>(theDict.GetCurrent());
				cout << "Entry number: " << cd.GetEntryNumber() << 
						" of " << cd.CountEntries() << endl;
				break;
			}
			case 'f':	// find word
			case 'F':
			{
				char word[256];
				
				cout << "Find: " << flush;
				cin.get(word, 255);
				cin.clear();
				cin.ignore(INT_MAX,'\n');
				
				theDict.FindEntry( word );
				if (theDict.WordFound())
					theHistory.RecordState(theDict);
				
				cout << endl;
				GetDefinition(theDict, formatter);
				break;
			}
			case 'm':	// make complex
			case 'M':
				char words[256] = "tarci sensi madzo";
				
				if (!CPrimSet::Good()) {
					cerr << "** Word making functions are not available. ***" << endl;
					break;
				}
				
				cout << "Make complex from: " << flush;
				cin.get(words, 255);
				cin.clear();
				cin.ignore(INT_MAX,'\n');
				
				{
					ComplexMaker wm(words);
					const char * result = wm.GetBestWord();
					const char * form = wm.GetBestForm();
					cout << "Best complex is: \"" << result << "\"" << endl;
					cout << "The form is: <" << form << ">" << endl;
					theDict.ChooseDictionary(LE);	// must use LE
					theDict.FindEntry( result );
				}
				
				GetDefinition(theDict, formatter);
				break;
				
			case 'h':	// history
			case 'H': {
					// display a list of items
					const HistoryStackType & stack = theHistory.GetStack();
					HistoryStackType::const_iterator i;
					int j;
					
					cout << "History:" << endl;
					for (i = stack.begin(), j=0; i!=stack.end(); i++, j++) {
						cout << "\t" << j << ") " << i->word << endl;
					}
					
					// ask which item to recall
					int which = 0;
					cout << "Which do you want to recall? : ";
					cin >> which;
					cin.clear();
					cin.ignore(INT_MAX,'\n');
					
					if (which >= 0 && which < theHistory.CountStates()) {
						theHistory.RecallState(theDict, which);
						GetDefinition(theDict, formatter);
					} else {
						cerr << "*** Your choice was out of range. ***" << endl;
					}
				}
				break;
				
			case 'w':	// web/HTML
			case 'W':
				cout << "Saving current word to test.html." << endl;
				htmlfile.open(htmlfilename);
				if ( htmlfile.is_open() ) {
					GetDefinition(theDict, html);
					htmlfile.close();
				} else {
					cerr << "*** Couldn't open file. ***" << htmlfilename << "." << endl;
				}
				break;
			
			case 'c':	// web/CGI
			case 'C': {
				cout << "Saving CGI reply to CGIResult.html." << endl;
				ofstream cgifile("CGIResult.html");
				
				if ( cgifile.is_open() ) {
					CCGIFormatter cgi(cgifile);
					cgi.SetServerName("Apple Personal Web Sharing");
					cgi.SetActionPath("http://www.loglan.org/LOD/LOD.cgi");
					cgi.SetHTMLTemplate(theCGIForm.c_str());
					
					cgi.SetWord(theDict.GetCurrent(), theDict.WhichDictionary());
					GetDefinition(theDict, cgi);
					cgifile.close();
				} else {
					cerr << "*** Couldn't open file. ***" << htmlfilename << "." << endl;
				}
				break;
			}
			case '!': {// parse (Not a regular feature. Just for testing.)
			//	PrintParse(theDict);	// the old way
				
				CFullParse theParse;
				theDict.GetCurrent().ParseEntry(theParse);
				cout << "Parsing current entry:" << endl;
				cout << "=====================" << endl;
				theParse.Print(cout);
				cout << "=====================" << endl;
				break;
			}
			case 'q': // quit
			case 'Q':
				looping = false; 
				break;
			default:
				cerr << "*** I didn't understand that. ***" << endl;
				break;
			}
			
		}
		
	} catch(...) {
		cerr << "*** Uncaught Exception ***" << endl;
		char ch;
		cin >> ch;	// pause before quitting
	}
	
	CPrimSet::Dispose();
	
	logdata.close();
	logindex.close();
	engdata.close();
	engindex.close();
	
	cout << "All done." << endl;
	
	return 0;
}

void GetDefinition( CLOD &theDict, CFormatter &formatter)
{
//	char s[25];
//	theDict.GetWord(s,25);
//	cout << "Getting " << s << ":" << endl;
	
	// exception safe formatting
//	formatter.BeginFormatting();
	try {
		theDict.GetDefinition(formatter);
//		formatter.EndFormatting();
	} catch (...) {
//		formatter.EndFormatting();
		cerr << "*** Unknown error while formatting. ***" << endl;
 	}
}

void PrintParse( CLOD &theDict )
{
	CEntryParser parser;
	const int len = 256;
	char s[len];
	
	theDict.GetCurrent().ParseEntry(parser);
	
	cout << "Parsing current entry:" << endl;
	cout << "=====================" << endl;
	parser.GetPart(part_Word, 0, s, len);
	cout << "Word: " << s << endl;
	if (theDict.WhichDictionary()==LE) {
		parser.GetPart(part_WordInfo, 0, s, len);
		cout << "WordInfo: " << s << endl;
			parser.GetPart(part_Affixes, 0, s, len);
			cout << "\tAffixes: " << s << endl;
			parser.GetPart(part_Etymology, 0, s, len);
			cout << "\tEtymology: " << s << endl;
		parser.GetPart(part_TypeField, 0, s, len);
		cout << "Type Field: " << s << endl;
			parser.GetPart(part_Percentage, 0, s, len);
			cout << "\tPercentage: " << s << endl;
			parser.GetPart(part_WordType, 0, s, len);
			cout << "\tWord Type: " << s << endl;
		parser.GetPart(part_Attribution, 0, s, len);
		cout << "Attribution: " << s << endl;
			parser.GetPart(part_Author, 0, s, len);
			cout << "\tAuthor: " << s << endl;
			parser.GetPart(part_Date, 0, s, len);
			cout << "\tDate: " << s << endl;
			parser.GetPart(part_Rank, 0, s, len);
			cout << "\tRank: " << s << endl;
	}
	long cnt = parser.CountDefinitions();
	for (int i=0; i<cnt && i<10 ; i++) {
		cout << "Definition " << i+1 << " of " << cnt << ":" << endl;
		parser.GetPart(part_Phrase, i, s, len);
		cout <<  "\tPhrase: " << s << endl;
		parser.GetPart(part_Grammar, i, s, len);
		cout <<  "\tGrammar: " << s << endl;
		parser.GetPart(part_DefinitionText, i, s, len);
		cout <<  "\tText: " << s << endl;
		if (theDict.WhichDictionary()==LE) {
			parser.GetPart(part_CaseTags, i, s, len);
			cout <<  "\tTags: " << s << endl;
		}
	}
	parser.GetPart(part_Footer, 0, s, len);	// sometimes truncated
	cout << "Footer: " << s << endl;
	cout << "=====================" << endl;
}

void EndianTest()
{	// a quick and dirty platform dependent test
	// not actually used for anything
	unsigned long n = 0x00010203;
	unsigned char * c = reinterpret_cast<unsigned char *>(&n) ;
	cout << "Endian Test: 0:" << c[0] << 
		" 1:" << c[1] << 
		" 2:" << c[2] << 
		" 3:" << c[3] << endl;
}

void CharacterScan(istream &is)
{
	// scan an istream, counting each kind of character
	unsigned long bin[256];
	
	// initialize
	for(int i=0; i<256; i++) bin[i] = 0;
	
	// count
	unsigned long cnt = 0;
	unsigned char ch;
	while(is.get((char&)ch)) {
		bin[ch]++;
		if ((++cnt % 100000) == 0) cout << '.' << flush;	// progress feedback
	}
	cout << endl;
	
	// report
	for (int i=0; i<256; i++) {
		if (bin[i]) {
			cout << i << hex << " 0x" << i << ' ' << (char)i << ' ' << dec << bin[i] << endl;
		}
	}
}

void CompareWordMakers(CLEDictionary &dict)
{
	// Compare the two word makers. (For testing.)
	// Also exercises the entry parser.
	CWordMaker wm;
	ComplexMaker cm;
	CFullParse p;
	string typ, word, ety, words;
	long cnt = 0;
	
	// rewind the dictionary
	dict.FirstEntry();
	// loop through all entries
	while (dict.CanDoNext()) {
		dict.NextEntry();	// note: we don't care about the first entry: not a cpx
		dict.ParseEntry(p);
		p.GetPart(typ, CParse::part_WordType);
		if (typ.find("Cpx",0) != string::npos) {	// is it a complex?
			p.GetPart(word, CParse::part_Word);
			p.GetPart(ety, CParse::part_Etymology);
			ComplexMaker::UnmangleForm(ety.c_str(), words);
	//		cout << word << ' ' << words << endl;
			wm.SetWords(words.c_str());
			cm.SetWords(words.c_str());
			if (wm.CanGetBest() && cm.CanGetBest()) {
				if (strcmp(word.c_str(), cm.GetBestWord()) != 0 || 
					strcmp(wm.GetBestWord(), cm.GetBestWord()) != 0) {
					// to do: detect redirections ("SEE") entries
					
					cerr << "mismatch: " << word.c_str() << ' ' << ety << ':' 
						 << wm.GetBestWord() << ':' << cm.GetBestWord() << ':'
						 << endl;
				}
			} else {
				cerr << "*** word maker failure on: " << word << endl;
			}
			cnt++;
		}
	//	if ( (++cnt)%500 == 0)
	//		cout << '(' << word.c_str() << ')' << endl;
		
	}
	cout << endl << "Processed " << cnt << " complexes." << endl;
}

void ParseDictionary(CDictionary &dict)
{
	// Run the entire dictionary through the parser
	// (For testing the parser.)
	CFullParse p;
	string word;
	long cnt;
	
	// rewind the dictionary
	dict.FirstEntry();
	// loop through all entries
	for (cnt = 0; dict.CanDoNext(); dict.NextEntry(), cnt++) {
		dict.ParseEntry(p);
		if (!p.Successful()) {
			p.GetPart(word, CParse::part_Word);
			cerr << "*** parse error in: " << word << endl;
		}
		if ( cnt%500 == 0 ) {
			p.GetPart(word, CParse::part_Word);
			cout << cnt << " " << word << endl;
		}
	}
	
	cout << "A total of " << cnt << " entries." << endl;
}
