/*
	CDictionaryIndex.h
	
	Index file for a Loglan dictionary.
	
	by James Jennings
	Started February 16, 1999
*/

#pragma once

#include <stdio.h>
#include <istream>
#include <assert.h>	// for debugging

using namespace std;

class CDictionaryIndex {
public:
			CDictionaryIndex( istream & index );
	
	// public interface
	bool	Find( const char *inWord, istream::pos_type &outPosition );
	bool	good() { return mIndex.good(); }
	
protected:
	enum { kSectorSize = 512, kMaxLevel = 3 };
	// accessors for interpreting mData in a platform independent way
	short 			GetChar(short i);
	short			GetJump(short i);
	short			GetSector(short i);
	unsigned long	GetAddress(short i);
	
	long	CleanUpInput( const char *in, char *out, int bufLen );
	
	void	LoadSector(short inSector);
	
	typedef enum { notFound, keepLooking, found } ESearchResult;
	ESearchResult Look( const unsigned char *key, short no );
	ESearchResult Search( const unsigned char *key, short sector, short level = 0 );
	
	unsigned char mData[ kSectorSize ];	// the currently loaded sector
	
	istream & mIndex;
	short	mRootSector, 
			mCurrentSector, 
			mNextSectorToSearch;
	unsigned long mFoundAddress,
			mNextAddress;
	bool	mFound;
};

/*
	We inline the methods that interpret the contents of mData.
	These are designed to be platform independent. They don't
	depend on big-endian or little-endian hardware.
*/

inline
short 	CDictionaryIndex::GetChar(short i)
{	// Fetch a character from the index file data.
	assert(i >= 0 && i < kSectorSize/2);
	
	return mData[ i*2 ];
}

inline
short	CDictionaryIndex::GetJump(short i)
{	// Fetch a jump within a sector from the index file data.
	assert(i >= 0 && i < kSectorSize/2);
	
	return mData[ i*2 + 1 ];
}

inline
short	CDictionaryIndex::GetSector(short i)
{	// Fetch a new sector number from the index file data.
	assert(i >= 0 && i < kSectorSize/2);
	
	// Note that we don't just typecast the pointer because 
	// we want this code to work on "big endian" hardware
	// with the same index files.
	i *= 2;
	unsigned short x = 	// reconstruct an integer
				(static_cast<unsigned short>(mData[i]) << 8) | 
				 static_cast<unsigned short>(mData[i+1]);
	return x >> 6;		// return the top 10 bits
}

inline
unsigned long	CDictionaryIndex::GetAddress(short i)
{	// Fetch a dictionary file offset from the index file data.
	assert(i >= 0 && i < kSectorSize/2 - 1);
	
	// Note that we don't just typecast the pointer because 
	// we want this code to work on "big endian" hardware
	// with the same index files.
	i *= 2;
	unsigned long x =
		//	static_cast<unsigned long>(mData[i  ]) << 24 | // masked out
			static_cast<unsigned long>(mData[i+1]) << 16 | 
			static_cast<unsigned long>(mData[i+2]) <<  8 | 
			static_cast<unsigned long>(mData[i+3]);
	// Return only 22 significant bits.
	// The remaining 10 bits is sector information.
	return x & 0x3fffff;
}

