/*
	CLOD.h
	
	The LOD fuctionality in one easy package.
	Basically just joins two dictionaries together.
	
	by James Jennings
	Started March 4, 1999
*/

#pragma once

#include "CELDictionary.h"
#include "CLEDictionary.h"
//#include "CWord.h"

struct CLOD; // forward reference

typedef enum { noDictionary, LE, EL } DictionaryType;

/*
	struct SLODState
	A struct for remembering the dictionary state.
	Used in CLODHistory.
*/
typedef struct SLODState {
		enum { kMaxWordLength = 25 };
		
		SLODState() : which(noDictionary) {}	// default constructor
		bool operator ==(const SLODState &rhs) const
				{ return which == rhs.which && state.current == rhs.state.current; }
		
		DictionaryType			which;
		SDictionaryState		state;
		char					word[kMaxWordLength+1];
		} SLODState;

class CLOD {
public:
		enum { kMaxWordLength = SLODState::kMaxWordLength };
	
					CLOD(istream &loddata, istream &lodindex, 
						 istream &engdata, istream &engindex);
	
	virtual void	ChooseDictionary(DictionaryType inType);
	virtual DictionaryType WhichDictionary() const { return mCurrentDictionary; }
	
	// save and restore the current dictionary state
	virtual void	GetState( SLODState &outState ) const;
	virtual void	SetState( const SLODState &inState );
	
	// get references to the dictionaries
	CDictionary			&GetCurrent();	// return the current dictionary
	const CDictionary	&GetCurrent() const;
	CLEDictionary		&GetLE() { return le; }
	const CLEDictionary	&GetLE() const { return le; }
	CELDictionary		&GetEL() { return el; }
	const CELDictionary	&GetEL() const { return el; }
	
	// for convenience: direct a number of methods to the current dictionary
	virtual bool FindEntry( const char *inWord ){ return GetCurrent().FindEntry(inWord); }
	virtual void NextEntry()				{ GetCurrent().NextEntry(); }
	virtual void PreviousEntry()			{ GetCurrent().PreviousEntry(); }
	virtual bool CanDoFind()				{ return GetCurrent().CanDoFind(); }
	virtual bool CanDoNext() const			{ return GetCurrent().CanDoNext(); }
	virtual bool CanDoPrevious() const		{ return GetCurrent().CanDoPrevious(); }
	virtual bool WordFound() const			{ return GetCurrent().WordFound(); }
	virtual void GetWord( char *outWord, size_t inBufLen = kMaxWordLength) const
											{ GetCurrent().GetWord(outWord,inBufLen); }
	virtual void GetDefinition( CFormatter &outText )
											{ GetCurrent().GetDefinition(outText); }
	
protected:
	CELDictionary					el;
	CLEDictionary					le;
	DictionaryType		mCurrentDictionary;
};

inline
CDictionary	&CLOD::GetCurrent()
{	// return the current dictionary
	assert(mCurrentDictionary == LE || mCurrentDictionary == EL);
	
	if (mCurrentDictionary == LE)
		return le;
	else
		return el;
}

inline
const CDictionary	&CLOD::GetCurrent() const
{	// return the current dictionary
	assert(mCurrentDictionary == LE || mCurrentDictionary == EL);
	
	if (mCurrentDictionary == LE)
		return le;
	else
		return el;
}

