/*
	CLODHistory.cp
	
	A class that tracks how a CLOD class has been used.
	
	by James Jennings
	March 22, 1999
*/

#include "CLODHistory.h"
#include "CLOD.h"

CLODHistory::CLODHistory( int inMaxEntries )
	: maxEntries(inMaxEntries)
{
}

void	CLODHistory::RecordState( const CLOD &lod )
{	// get the current LOD state and save it in the stack

	SLODState state;
	lod.GetState(state);
	
	// check for duplicates
	HistoryStackType::iterator i = find(stack.begin(), stack.end(), state);
	if (i != stack.end())
		stack.erase(i);	// it was already there: pop it to the front
	
	stack.push_front(state);
	
	// purge old records
	while(stack.size() > maxEntries)
		stack.pop_back();
}

void	CLODHistory::RecallState( CLOD &lod, int index )
{	// restore the dictionary to the specified saved state
	
	assert(0<=index && index < stack.size());
	
	const SLODState & state = GetState(index);
	lod.SetState(state);
	
	// to do: move the recalled state to the front
}

int		CLODHistory::CountStates()
{
	return stack.size();
}

const SLODState & CLODHistory::GetState(int index)
{
	assert(index>=0 && index < stack.size());
	return stack[index];
}


const HistoryStackType & CLODHistory::GetStack()
{
	return stack;
}

