/*
	CCGIFormatter.h
	
	A formatter for generating a web server/CGI response.
	
	James Jennings * March 24, 2000
*/

#pragma once

#include "CHTMLFormatter.h"
#include "CLOD.h"

enum {	// common HTTP status codes
	// 2xx Success
	httpOK 					= 200,
	httpNoContent 			= 204,
	// 3xx Redirection
	httpMovedTemporarily 	= 302,
	httpNotModified 		= 304,
	// 4xx Client error
	httpUnauthorized 		= 401,
	httpNotFound 			= 404,
	// 5xx Server error
	httpInternalServerError = 500,
	httpNotImplemented 		= 501

};

class CCGIFormatter : public CHTMLFormatter {
public:
//	typedef CLOD::DictionaryType	DictionaryType;
	
	CCGIFormatter( ostream &os = cout );
	
	virtual void SetServerName( const char *server );
	virtual void SetActionPath( const char *path );
	virtual void SetHTMLTemplate( const char *temp );
	 	// pull current word info from dictionary
	virtual void SetWord( CDictionary & theDict, DictionaryType which );
	
	virtual void BeginFormatting();
	virtual void EndFormatting();
	
	virtual void SendStartingPage();
	
protected:
	virtual void PutHTMLHeader( int type );
	
	// template manipulations
	virtual void Replace( string &s, const char *oldText, const char *newText);
	virtual void MakeLookupAnchor(string &result, const char *word, DictionaryType whichDictionary);
	virtual void ProcessTemplate();
	
	string 	serverName;		// the name of the server we're running on
	string 	actionPath;		// the url to this CGI script
	string 	htmlTemplate;	// The HTML form we need to complete.
	string	currentWord, previousWord, nextWord;
	DictionaryType	whichDictionary;
	bool	wordFound;
	
	string header, footer;		// scratch variables: two halves of the munged template
};
