/*
	CFormatter.h
	
	Pure abstract class that a CDictionary uses to format an entry.
	
	by James Jennings
	Started February 15, 1999
*/

#pragma once

// EStyle are abstract style types that are used in a Loglan dictionary entry.
// Subclasses of CFormatter will interpret these codes according to their abilities.
// (The best selection of these styles is still in flux.)
typedef enum {
			style_Normal,
			style_Error,		// red
			style_Word,			// bold
			style_Header,		// green italic
			style_DefinitionName,// bold
			style_Definition,
			style_Field,
			style_Translation	// blue italic
			// etc.
			} EStyle;

/*
	CFormatter:
	Abstract base class to encapsulate platform and target dependent code.
	A "helper" class for CDictionary.
	Possible subclasses:
		COStreamFormatter: Sends text to an ostream. Used in DOS and Unix implimentations.
		CHTMLFormatter: Formats with HTML tags. Used in web server CGIs.
		CTEFormatter: Formats text in a Macintosh TERecord. Used in Mac applications.
		CAEFormatter: Formats text for an AppleEvent Reply.
		CWasteFormatter: Formats text in a Waste Record. Used in Mac applications.
		CWin95Formatter: Formats text in whatever works in Windows 95.
*/
class CFormatter {
public:
	virtual void BeginFormatting() {}
	virtual void SetStyle( EStyle /*inStyle*/ ) {}
	virtual void PutChar( char inChar ) = 0;
	virtual void PutString( char *inString ) = 0;
	virtual void PutNewLine() = 0;
	virtual void EndFormatting() {}
	virtual void AbortFormatting() { EndFormatting(); }
};

