/*
	CHTMLFormatter.cp
	
	A class for formatting an LOD entry as HTML.
	
	by James Jennings
	Started February 19, 1999
	
*/

#include "CHTMLFormatter.h"
#include <ctype.h>

CHTMLFormatter::CHTMLFormatter(ostream &os)
	: out(os), mCurrent(style_Normal), mLast('\0')
{
}

void CHTMLFormatter::BeginFormatting()
{
	mCurrent = style_Normal;
	
	out << "<HTML>" << endl;
	out << "<HEAD>" << endl;
	out << "<TITLE>Loglan Online Dictioary</TITLE>" << endl;
	out << "</HEAD>" << endl;
	out << "<BODY bgcolor=\"#FFFFFF\">" << endl;
}

void CHTMLFormatter::SetStyle( EStyle inStyle )
{
	if (inStyle == mCurrent) return;	// nothing to change
	
	// end the old style
	switch (mCurrent) {
	case style_Normal:
		break;
	case style_Error:		// red
		out << "</FONT>";
		break;
	case style_Word:		// bold
		out << "</B>";
		break;
	case style_Header:		// medium green italic
		out << "</EM>" << "</FONT>";
		break;
	case style_DefinitionName:// bold
		out << "</B>";
		break;
	case style_Definition:
		break;
	case style_Field:
		break;
	case style_Translation:	// blue italic
		out << "</EM>" << "</FONT>";
		break;
	}

	// start the new style
	switch (inStyle) {
	case style_Normal:
		break;
	case style_Error:		// red
		out << "<FONT COLOR=\"#FF0000\">";
		break;
	case style_Word:		// bold
		out << "<B>";
		break;
	case style_Header:		// medium green italic
		out << "<FONT COLOR=\"#007F00\">" << "<EM>";
		break;
	case style_DefinitionName:// bold
		out << "<B>";
		break;
	case style_Definition:
		break;
	case style_Field:
		break;
	case style_Translation:	// blue italic
		out << "<FONT COLOR=\"#0000FF\">" << "<EM>";
		break;
	}
	
	// remember the new style
	mCurrent = inStyle;
}

void CHTMLFormatter::PutChar( char inChar )
{
	// some Mac-only characters
	// (defined to make this code platform independent)
// MK: added unsigned
/*	const unsigned char
			OUmlaut		= 0x85,	//  
			aUmlaut		= 0x8a,	//  
			ccedilla	= 0x8d,	//  
			eAcute		= 0x8e,	//  
			eGrave		= 0x8f,	//  
			nTilde		= 0x96,	//  
			oUmlaut		= 0x9a,	//  
			uUmlaut		= 0x9f,	//  
			degree		= 0xa1,	//  
			pound		= 0xa3,	//  
			sharps		= 0xa7,	//  small sharp s, German, sz ligature
			umlaut		= 0xac,	//  
			del			= 0xb6,	//  
			oSlash		= 0xbf,	//  
			fortisimo	= 0xc4,	//  
			ellipsis	= 0xc9,	//  	ellipsis
			nbsp		= 0xca,	//  
			endash		= 0xd0,	//  	endash
			emdash		= 0xd1,	//  	emdash
			rDblQuote	= 0xd3,	//  
			rSngQuote	= 0xd5;	//  
*/				// more?
	
	// Handle platform dependant 8-bit characters.
	if ( (inChar&0x80) != 0 ) {
		if (entity[inChar & 0x7F] != 0) 
			out << entity[inChar & 0x7F];
		else
			out << inChar << "[bad char?]";
	} else {
		switch((unsigned char)inChar) {
		
		// Handle special Mac-only characters.
/*		case ellipsis:		out << "...";		break;
		case emdash:		out << "--";		break;
		case endash:		out << "-";			break;
		case nbsp:			out << "&nbsp;";	break;
		case rDblQuote:		out << "\"";		break;	//  
		case rSngQuote:		out << "\'";		break;	//  
		
		case OUmlaut:		out << "&Ouml;";	break;	//  
		case aUmlaut:		out << "&auml;";	break;	//  
		case ccedilla:		out << "&ccedil;";	break;	//  
		case eAcute:		out << "&eacute;";	break;	//  
		case eGrave:		out << "&egrave;";	break;	//  
		case nTilde:		out << "&ntilde;";	break;	//  
		case oUmlaut:		out << "&ouml;";	break;	//  
		case uUmlaut:		out << "&uuml;";	break;	//  
		case degree:		out << "&deg;";		break;	//  
		case pound:			out << "&pound;";	break;	//  
		case sharps:		out << "&szlig;";	break;	//  
		case oSlash:		out << "&oslash;";	break;	//  
		case fortisimo:		out << "f";	break;	//  
*/
		// Handle special HTML characters
		case '>': out << "&gt;";	break;
		case '<': out << "&lt;";	break;
		case '&': out << "&amp;";	break;
		
		// multiple white space gets collapsed unless we do something
		case ' ':
			if (isspace(mLast)) out << "&nbsp;";
			else out << inChar;
			break;
			
		default:
			// everything else is sent straight
			assert( (inChar & 0x80) == 0 /* no 8-bit chars allowed */);
			out << inChar;
			break;
		}
	}
	
	mLast = inChar;
}

void CHTMLFormatter::PutString( char *inString )
{
	while( *inString != '\0' ) 
		PutChar( *(inString++) );
}

void CHTMLFormatter::PutNewLine()
{
	out << "<BR>" << endl;
}

void CHTMLFormatter::EndFormatting()
{
	SetStyle(style_Normal);
	out << "</BODY>" << endl;
	out << "</HTML>" << endl;
}

// A lookup table of HTML entity codes for the Macintosh character set.
// Count begins at 0x80. '0' means that character is not available,
// either because it isn't Latin 1, or because we're using it for other things.
// 'reserved' means reserved for LOD use
// 'notdefined' means no escape code known
#define reserved	0
#define notdefined	0
const char * CHTMLFormatter::entity[] = {
			"&Auml;",	// 0x80 
			"&Aring;",	// 0x81 
			"&Ccedil;",	// 0x82 
			"&Eacute;",	// 0x83 
			"&Ntilde;",	// 0x84 
			"&Ouml;",	// 0x85 	used in LOD
			"&Uuml;",	// 0x86 
			"&aacute;",	// 0x87 
			"&agrave;",	// 0x88 
			"&acirc;",	// 0x89 
			"&auml;",	// 0x8a 	used in LOD
			"&atilde;",	// 0x8b 
			"&aring;",	// 0x8c 
			"&ccedil;",	// 0x8d 	used in LOD
			"&eacute;",	// 0x8e 	used in LOD
			"&egrave;",	// 0x8f 	used in LOD
			"&acirc;",	// 0x90 
			"&euml;",	// 0x91 
			"&igrave;",	// 0x92 
			"&iacute;",	// 0x93 
			"&icirc;",	// 0x94 
			"&iuml;",	// 0x95 
			"&ntilde;",	// 0x96 	used in LOD
			"&oacute;",	// 0x97 
			"&ograve;",	// 0x98 
			"&ocirc;",	// 0x99 
			"&ouml;",	// 0x9a 	used in LOD
			"&otilde;",	// 0x9b 
			"&uacute;",	// 0x9c 
			"&ugrave;",	// 0x9d 
			"&ucirc;",	// 0x9e 
			"&uuml;",	// 0x9f 	used in LOD
			notdefined,	// 0xa0 	dagger
			"&deg;",	// 0xa1 	used in LOD
			"&cent;",	// 0xa2 
			"&pound;",	// 0xa3 	used in LOD
			"&sect;",	// 0xa4 	section mark
			notdefined,	// 0xa5 	bullet
			"&para;",	// 0xa6 	paragraph mark
			"&szlig;",	// 0xa7  small sharp s, German, sz ligature, used in LOD
			"&reg;",	// 0xa8 	registered
			"&copy;",	// 0xa9 	copyright
			"(tm)",		// 0xaa 	trademark
			"&acute;",	// 0xab 
			"&uml;",	// 0xac  	umlaut	used in LOD
			"&not;",	// 0xad 
			"&AElig;",	// 0xae  	capital AE diphthong (ligature)
			"&Oslash;",	// 0xaf 
			notdefined,	// 0xb0 	infinity
			"&plusmn;",	// 0xb1 
			reserved,	// 0xb2 	less than or equals
			reserved,	// 0xb3 	greater than or equal
			"&yen;",	// 0xb4 
			"&micro;",	// 0xb5 	greek mu
			notdefined,	// 0xb6 	small delta	used in LOD	(why?)
			notdefined,	// 0xb7 	capital Sigma
			notdefined,	// 0xb8 	capital Pi
			notdefined,	// 0xb9 	small pi
			notdefined,	// 0xba 	integral sign
			"&ordf;",	// 0xbb 
			"&ordm;",	// 0xbc 
			notdefined,	// 0xbd 	capital Omega
			"&aelig;",	// 0xbe  small AE diphthong (ligature)
			"&oslash;",	// 0xbf 	used in LOD
			"&iquest;",	// 0xc0 	inverted question
			"&iexcl;",	// 0xc1 	inverted explaination mark
			notdefined,	// 0xc2 	bent dash
			notdefined,	// 0xc3 	soludus/square root
			notdefined,	// 0xc4 	script f	used in LOD	(why?)
			notdefined,	// 0xc5 	approx. equal
			notdefined,	// 0xc6 	capital Delta
			reserved,	// 0xc7 	left french quote
			reserved,	// 0xc8 	right french quote
			"...",		// 0xc9 	ellipsis	used in LOD
			"&nbsp;",	// 0xca 	non-breaking space	used in LOD
			"&Agrave;",	// 0xcb 
			"&Atilde;",	// 0xcc 
			"&Otilde;",	// 0xcd 
			"&OElig;",	// 0xce 	(guessing code)
			"&oelig;",	// 0xcf 	(guessing code)
			"-",		// 0xd0  	endash	used in LOD
			"--",		// 0xd1  	emdash	used in LOD
			"&quot;",	// 0xd2 	left curly quote
			"&quot;",	// 0xd3 	right curly quote	used in LOD
			"'",		// 0xd4 
			"'",		// 0xd5 	used in LOD
			reserved,	// 0xd6 	divide
			notdefined,	// 0xd7 	diamond
			"&yuml;",	// 0xd8 
			"&Yuml;",	// 0xd9 
			"/",		// 0xda 	deep slash (for fractions?)
			"&curren;",	// 0xdb 	symbol for the euro currency
			notdefined,	// 0xdc 	left french single quote
			notdefined,	// 0xdd 	right french single quote
			"fi",		// 0xde 	fi ligature
			"fl",		// 0xdf 	fl ligature
			notdefined,	// 0xe0 	double dagger
			"&middot;",	// 0xe1 	middle dot, small bullet
			notdefined,	// 0xe2 	comma
			notdefined,	// 0xe3 	low close quote
			"0/00",		// 0xe4 	0/00, fancy percent, "an no cents"
			"&Aring;",	// 0xe5 
			"&Ecirc;",	// 0xe6 
			"&Aacute;",	// 0xe7 
			"&Euml;",	// 0xe8 
			"&Egrave;",	// 0xe9 
			"&Iacute;",	// 0xea 
			"&Icirc;",	// 0xeb 
			"&Iuml;",	// 0xec 
			"&Igrave;",	// 0xed 
			"&Oacute;",	// 0xee 
			"&Ocirc;",	// 0xef 
			notdefined,	// 0xf0 	apple logo
			"&Ograve;",	// 0xf1 
			"&Uacute;",	// 0xf2 
			"&Ucirc;",	// 0xf3 
			"&Ugrave;",	// 0xf4 
			notdefined,	// 0xf5 	short vertical bar
			notdefined,	// 0xf6 	circumflex
			notdefined,	// 0xf7 	tilde
			"&hibar;",	// 0xf8 
			notdefined,	// 0xf9 	short vowel mark
			notdefined,	// 0xfa 	high dot
			notdefined,	// 0xfb 	small diamond
			notdefined,	// 0xfc		
			"&quot;",	// 0xfd  inch mark?
			notdefined,	// 0xfe 	
			notdefined	// 0xff 	inverted circumflex
	};
