/*
	COStreamFormatter.h
	
	Class that formats a Loglan dictionary entry for a C++ ostream.
	Defaults to sending to stdout.
	Styles are ignored.
	Used in DOS and Unix implimentations.
	
	by James Jennings
	Started February 15, 1999
*/

#pragma once

//#include <stdio.h>
#include "CFormatter.h"

class COStreamFormatter : public CFormatter {
public:
	COStreamFormatter( ostream & out = cout ) : out(out) {}

	virtual void PutChar( char inChar );
	virtual void PutString( char *inString );
	virtual void PutNewLine() { out << endl; }
	virtual void EndFormatting() { out << endl; }	// flush buffers
protected:
	ostream &out;
};

inline
void COStreamFormatter::PutChar( char inChar )
{
	// some Mac-only characters
	// (defined to make this code platform independent)
// MK: added unsigned
	const unsigned char
				ellipsis = 0xC9,	// ''
				emdash = 0xD1,		// ''
				endash = 0xD0;		// ''
	
	switch((unsigned char)inChar) {
	
	// Handle special Mac-only characters.
	// (We might want to conditionally compile this
	//  to use platform specific characters.)
	case ellipsis: out << "..."; break;
	case emdash:   out << "--"; break;
	case endash:   out << "-"; break;
	
	default:
		// everything else is sent straight
	//	assert( (inChar & 0x80) == 0 /* no 8-bit chars allowed */);
		out << inChar;
		if ((inChar & 0x80) != 0) out << "[?]"; /* flag 8-bit characters */
		break;
	}
}

inline
void COStreamFormatter::PutString( char *inString )
{
	while( *inString != '\0' ) 
		PutChar( *(inString++) );
}
