/*
	CFullParse.cpp
	
	Record everything it's told in a straightforward manner.
	
	James Jennings * April 7, 2000
*/

#include "CFullParse.h"
#include <assert.h>

void 	CFullParse::BeginParsing()
{
	text.clear();
	styles.clear();
	parts.clear();
	definitionCount = 0;
	success = true;
}

void 	CFullParse::SetStyle( EStyle inStyle )
{
	StyleLocation loc( inStyle, text.size() );
	styles.push_back( loc );
}

void 	CFullParse::SetPart( EPart inPart, EPartEdge inEdge )
{
	short index = (IsDefinitionPart(inPart) ? definitionCount : 0 );
	PartLocation loc( inPart, inEdge, index, text.size() );
	parts.push_back( loc );
	if (inPart == part_Definition && inEdge == part_End) definitionCount++;
}

void 	CFullParse::PutChar( char inChar )
{
	text.push_back(inChar);
}

void 	CFullParse::PutString( const char *inString )
{
	assert(inString != 0);
	while (*inString!='\0') {
		PutChar(*inString);
		inString++;
	}
}

void 	CFullParse::PutNewLine()
{
	PutChar('\n');
}

void 	CFullParse::PutIndent()
{
	PutString("   ");
}

/*
void 	CFullParse::EndParsing()
{
	
}
*/

bool CFullParse::IsDefinitionPart( EPart inPart )
{
	return ( part_FirstDefinitionPart <= inPart && 
			 inPart < part_NoMoreDefinitionParts ) || 
			inPart == part_Definition;
}

bool CFullParse::GetPartRange( EPart which, short index, pos_type &start, pos_type &stop) const
{
	// find the range of characters occupied by the part
	// returns false if not found
	start = stop = string::npos;

/*	
	PartList::iterator i = parts.begin();
	// find the nth part start
	for ( ; index>=0 ; index--) {
		for ( ; i!=parts.end(); i++) 
			if ( i->part == which && i->edge == part_Start ) break;
		if (i == parts.end()) return false;
		if (index == 0) break;
		i++;
	}
	start = i->where;
	
	// find the next part end
	PartList::iterator j = i;
	for ( ; j!=parts.end(); j++) 
		if ( j->part == which && j->edge == part_End ) break;
	if (j == parts.end()) return false;
	
	stop = j->where;
*/
	PartLocation key(which, part_Start, index, 0);
	PartList::const_iterator i = find(parts.begin(), parts.end(), key);
	if ( i == parts.end() ) return false;
	start = i->where;
	
	key.edge = part_End;
	i = find(i, parts.end(), key);
	if ( i == parts.end() ) return false;
	stop = i->where;
		
	return true;
}

bool CFullParse::HasPart( EPart which, short index ) const
{
	pos_type start, stop;
	return GetPartRange(which, index, start, stop);
}

long CFullParse::GetPartLength( EPart which, short index ) const
{
	pos_type start, stop;
	
	if (GetPartRange(which, index, start, stop))
		return stop - start;
	
	return 0;
}

const char * CFullParse::GetPart( string &out, EPart which, short index ) const
{
	out.clear();
	
	pos_type start, stop;
	if ( GetPartRange(which, index, start, stop) ) {
		out.assign(text, start, stop-start);
	}
	
	return out.c_str();
}

const unsigned char * CFullParse::GetPart( unsigned char out[256], EPart which, short index) const
{
	// return the result as a 256 char Pascal string
	out[0] = 0;
	
	pos_type start, stop;
	if ( GetPartRange(which, index, start, stop) ) {
		int len = stop-start;
		if (len > 255) len = 255;
		::memmove(out+1, text.c_str() + start, len);
		out[0] = len;
	}
	
	return out;
}

short	CFullParse::CountDefinitions()
{
	return definitionCount;
}

void	CFullParse::Print(ostream &os)
{
	string scratch;
	
	os << "Text of the full parse: " << endl << text << endl;
//	os << "Numbers: " << styles.size() << ' ' << parts.size() << endl;
	os << "Word: " << GetPart(scratch, part_Word) << endl;
	os << "Header: " << GetPart(scratch, part_Header) << endl;
	if (HasPart(part_Redirect)) {
		os << "\tRedirect to: " << GetPart(scratch, part_Redirect) << endl;
		return;
	}
	os << "\tWord Info: " << GetPart(scratch, part_WordInfo) << endl;
		os << "\t\tAffixes: " << GetPart(scratch, part_Affixes) << endl;
		os << "\t\tEtymology: " << GetPart(scratch, part_Etymology) << endl;
	os << "\tType Field: " << GetPart(scratch, part_TypeField) << endl;
		os << "\t\tPercentage: " << GetPart(scratch, part_Percentage) << endl;
		os << "\t\tWord Type: " << GetPart(scratch, part_WordType) << endl;
	os << "\tAttribution: " << GetPart(scratch, part_Attribution) << endl;
		os << "\t\tAuthor: " << GetPart(scratch, part_Author) << endl;
		os << "\t\tDate: " << GetPart(scratch, part_Date) << endl;
		os << "\t\tRank: " << GetPart(scratch, part_Rank) << endl;
	
	for (int i=0; HasPart(part_Definition, i); i++) {
		os << "Definition#" << i+1 << ": " << GetPart(scratch, part_Definition, i) << endl;
		os << "\tPhrase:  " << GetPart(scratch, part_Phrase, i) << endl;
		os << "\tGrammar: " << GetPart(scratch, part_Grammar, i) << endl;
		os << "\tText:    " << GetPart(scratch, part_DefinitionText, i) << endl;
		os << "\tTags:    " << GetPart(scratch, part_CaseTags, i) << endl;
	}
	if (HasPart(part_Footer))
		os << "Footer: " << GetPart(scratch, part_Footer) << endl;
	
}
