/*
	CFullParse.h
	
	Record everything it's told in a straightforward manner.
	
	James Jennings * April 7, 2000
*/

#pragma once

#include "CParse.h"

#include <string>
#include <vector>
#include <ostream>	// for debugging
#include <iostream> // for debugging
using namespace std;

class CFullParse : public CParse {
protected:
	// typedefs for this class
	typedef string::size_type pos_type;
	class StyleLocation {
	public:
		StyleLocation( EStyle s = style_Normal, pos_type w = 0 ) 
			: style(s), where(w) {}
		EStyle		style;
		pos_type	where;
	};
	typedef vector<StyleLocation> StyleList;
	class PartLocation {
	public:
		PartLocation() 
			: part(part_NoMoreParts), edge(part_Start), index(0), where(0) {}
		PartLocation( EPart s, EPartEdge e, short i, pos_type w ) 
			: part(s), edge(e), index(i), where(w) {}
		bool operator == (const PartLocation &rhs) const
			{ return part==rhs.part && edge==rhs.edge && index==rhs.index; }
		EPart 		part;
		EPartEdge	edge;
		short		index;
		pos_type 	where;
	};
	typedef vector<PartLocation> PartList;
public:
	
	virtual void 	BeginParsing();
	virtual void 	SetStyle( EStyle inStyle );
	virtual void 	SetPart( EPart inPart, EPartEdge inEdge );
	virtual void 	PutChar( char inChar );
	virtual void 	PutString( const char *inString );
	virtual void 	PutNewLine();
	virtual void 	PutIndent();
//	virtual void 	EndParsing();
	virtual void 	AbortParsing() { success = false; }
	
	virtual bool	Successful() { return success; }
	
	virtual bool		 HasPart( EPart which, short index = 0 ) const;
	virtual long		 GetPartLength( EPart which, short index = 0 ) const;
	virtual const char * GetPart( string &out, EPart which, short index = 0) const;
	virtual const unsigned char * GetPart( unsigned char out[256], EPart which, short index = 0) const;
	virtual short		CountDefinitions();
	
	virtual void	Print(ostream &os);	// for debugging
protected:
	virtual bool	GetPartRange( EPart which, short index, pos_type &start, pos_type &stop) const;
	virtual bool	IsDefinitionPart( EPart which );
	
	string 		text;
	StyleList	styles;
	PartList	parts;
	short		definitionCount;
	
	bool		success;
};

/*
inline
void CFullParse::AbortParsing()
{
	string w;
	GetPart(w, CParse::part_Word);
	cerr << "*** parse error in: " << w << endl;
	success = false;
}
*/
