/*
	CParse.h
	
	An abstract class receives the actions performed by CParser.
	Based on CFormatter.
	
	James Jennings * April 7, 2000
*/

#pragma once

class CParse {
public:
/*
	CParse::EStyle
	Abstract style types that are used in a Loglan dictionary entry.
	Subclasses of CParse will interpret these codes according to their abilities.
*/
typedef enum {
			style_Normal,
			style_Error,		// red
			style_Word,			// bold
			style_Origins,		// green italic (in header)
			style_LoglanText,	// bold
			style_Metaphor,		// blue italic (in definitions)
			style_Translation	// blue italic
			// etc.
			} EStyle;
/*
	CParse::EParts
	Codes for the meaningful parts of an entry
*/
typedef enum {
		// 1st level parse
		part_Word,		// the word the entry is for
		part_Header,	// meta information about the word (LE dictionary only)
		part_Definition,// a definition (may be more than one)
		part_Footer,
		// 2nd level parse
		part_FirstHeaderPart,
			part_WordInfo = part_FirstHeaderPart,		// affixes, etymology
			part_Unused,
			part_TypeField,		// Eg: "C-Prim"
			part_Attribution,	// Author, Date, Rank
			part_Redirect,		// only in SEE redirections
		part_NoMoreHeaderParts,
		part_FirstDefinitionPart,
			part_Phrase,		// eg: 'lopo --'
			part_Grammar,		// eg: '(3a)'
			part_DefinitionText,
			part_CaseTags,		// eg: '[B-D]'
			part_Metaphor,		// (EL dictionary only)
		part_NoMoreDefinitionParts,
		// 3rd level parse (header only)
		part_FirstWordInfoPart,
			part_Affixes,
			part_Etymology,
		part_FirstTypeFieldPart,
			part_Percentage,
			part_WordType,
		part_FirstAttributionPart,
			part_Author,		// eg: 'L4' or 'JCB'
			part_Date,			// eg: "'77"
			part_Rank,			// Eaton rank, eg: '2.4'
		//
		part_NoMoreParts
		} EPart;
	typedef enum { part_Start, part_End } EPartEdge;
/*
	Methods used to build the parse.
*/
public:
	virtual void 	BeginParsing() {}
	virtual void 	SetStyle( EStyle /*inStyle*/ ) {}
	virtual void 	SetPart( EPart /*inPart*/, EPartEdge /*inEdge*/ ) {}
	virtual void 	PutChar( char inChar ) = 0;
	virtual void 	PutString( const char *inString ) = 0;
	virtual void 	PutNewLine() = 0;
	virtual void 	PutIndent() = 0;
	virtual void 	EndParsing() {}
	virtual void 	AbortParsing() { EndParsing(); }
};
