/*
	CParser.h
	
	A class that encapsulates all we need to know about parsing an LOD record.
	It can also be used for more sophisticated formatting.
	
	James Jennings * April 7, 2000
*/

#pragma once

#include "CParse.h"
#include <istream>
using namespace std;

class CParser {
public:
/*
		List of characters that have special meanings.
*/
enum {
//	char_HeaderDelimiter	= '|',
	char_StartEtymology		= '<',
	char_EndEtymology		= '>',
	char_AttributionDelimeter = ' ',
//	char_StartPhrase		= 0xB2, 	// '' Mac less than or equals
//	char_StartGrammar		= 0xB3,		// '' Mac greater than or equals
//	char_DefinitionDelimeter= '|',
	char_StartTag			= '[',
	char_EndTag				= ']',
	
	char_EndOfEntry			= 0x0A,		//	LF
	char_EndOfEntry2		= 0x0D,		//	CR	not actually used
	char_EndOfLine			= 0xB2,		// '' Mac less than or equals,
	char_EndHeader			= 0xB2, 	// '' Mac less than or equals,
	char_EndDefinition		= 0xB2, 	// '' Mac less than or equals,
	char_EndFooter			= 0xB2, 	// '' Mac less than or equals,
	char_EndField			= '|',
	char_EndWord			= ',',		//
	char_EndPhrase			= 0xB3,		// '' Mac greater than or equals
	
	char_StyleLoglan		= 0xC7,		// '' Mac left French quote
	char_EndStyleLoglan		= 0xC8,		// '' Mac right French quote
	char_StyleEmphasis		= 0xD6,		// '' Mac divide
	char_EndStyleEmphasis	= 0xC0		// '' Mac Spanish left question mark
	};

		CParser( istream &inIS, CParse &inParse );
	
	virtual void	ParseEntry();
protected:
	// recursive descent parser calls
	// Level 1:
	virtual bool	Word();
	virtual bool	Header();
	virtual bool	Definition();
	virtual bool	Footer();
	// Level 2: Header
	virtual bool	WordInfo();
	virtual bool	TypeField();
	virtual bool	Attribution();
	// Level 2: Definition
	virtual bool	Phrase();
	virtual bool	Grammar();
	virtual bool	DefinitionText();
	virtual bool	CaseTags();
	virtual bool	Metaphor();
	// Level 3: Header, WordInfo
	virtual bool	Affixes();
	virtual bool	Etymology();
	// Level 3: Header, TypeField
	virtual bool	Percentage();
	virtual bool	WordType();
	// Level 3: Header, Attribution
	virtual bool	Author();
	virtual bool	Date();
	virtual bool	Rank();
	
	// repeat the CParse interface so we can fuss things a bit on their way out
	virtual void 	SetStyle( CParse::EStyle inStyle ) { parse.SetStyle(inStyle); }
	virtual void 	SetPart( CParse::EPart /*inPart*/, CParse::EPartEdge /*inEdge*/ );
	virtual void 	PutChar( char inChar );
	virtual void 	PutString( const char *inString ) { parse.PutString(inString); } 
	virtual void 	PutNewLine() { parse.PutNewLine(); }
	virtual void 	PutIndent() { parse.PutIndent(); }
	
	// utility functions
	virtual bool	PutUntil( char stopChar );
	virtual void	SkipWhiteSpace();
	virtual bool	isLE() { return ! headerIsEmpty; }
	
	// Exceptions
	class ParseFailedException	: public exception {};
	class EndOfFieldException	: public exception {};
	class EndOfLineException	: public EndOfFieldException {};
	class EndOfEntryException	: public EndOfLineException {};

protected:
	istream &in;
	CParse	&parse;
	
	// some useful state variables
	bool doingHeader;
	bool doingDefinition;
	bool headerIsEmpty;
	bool redirected;
};
