/*
	CPrimSet.h
	
	A class for using the prim.set data.
	
	by James Jennings
*/

#pragma once

#include <istream>
#include <string.h>
#include <assert.h>

using namespace std;

class CPrim {
	// Remember a single primitive and it's affixes.
	friend istream & operator >> (istream &is, CPrim &prim);
public:
	enum { kPrimLength = 5, kAffixLength = 3, kMaxCount = 3, kFormLength = 10 };
	
				CPrim();		// default constructor
	
	// accessors
	const char * Word() const		{ return word; }
	const char * Affix(int i) const { assert(i>=0 && i<count); return aff[i]; }
	short		 AffixCount() const { return count; }
	
	const char * GetAffixForm( int i, char outForm[kFormLength+1] ) const;
	
//	bool operator == ( CPrim &rhs ) { return strcmp(word, rhs.word) == 0; }
//	bool operator < ( CPrim &rhs ) { return strcmp(word, rhs.word) < 0; }
//	bool operator > ( CPrim &rhs ) { return strcmp(word, rhs.word) > 0; }
	
protected:
	char 	word[kPrimLength+1];				// The primitive
	short	count;								// the number of affices
	char 	aff[kMaxCount][kAffixLength+1];		// affix list
};
 	

class CPrimSet {
	// Remember a list of CPrims.
	// This is a "singleton" object: we only allow exactly one to exist.
	// Don't construct: call Initialize instead.
protected:
							CPrimSet(istream &is);
	virtual					~CPrimSet();
public:
	static void				Initialize(istream &is);
	static void				Dispose();
	
	static const CPrim * 	Find( const char *word );
	static bool				Good() { return sSingleton != 0 && sSingleton->count>0; }
	
protected:
	virtual const CPrim *	FindWord( const char *word ) const;
	
	short count;
	CPrim * primList;
	
	static CPrimSet * sSingleton;
};
