/*
	ComplexMaker.h
	
	class ComplexMaker
	
	For making complex predicates.
	
	James Jennings * April 19, 2000
*/

#pragma once

#include "Affix.h"
#include <vector>
#include <ostream>

using namespace std;

class ComplexMaker {
public:
//	static bool 	IsBorrowed(const char *theString);
	static bool 	IsBadCccJoint(char a, char b, char c);
	static bool 	IsValidJunction(char a, char b);
	static bool  	IsInitialPair(char a, char b);

	static const char *	UnmangleForm( const char inForm[], string &out );

							ComplexMaker();
							ComplexMaker(const char *words);
							ComplexMaker(const unsigned char *words);
							
	virtual void			Clear();
	virtual void			SetWords( const char *wds );
	virtual bool			CanGetBest();
	virtual const char *	GetBestWord() { return bestWord.c_str(); }
	virtual const char *	GetBestForm() { return bestForm.c_str(); }
	
	virtual int				GetComplexity();
	
	virtual ostream & debug(ostream &os);
	
protected:
	typedef vector<Word>		WordList;
	typedef vector<AffixSet> 	AffixList;
	typedef vector<int>			AffixPermutation;
	
	virtual void			FindBest();
	virtual	void			MakeWord(const AffixPermutation &p);
	virtual bool			NextPermutation(AffixPermutation &p);
	
	WordList	words;
	AffixList	affix;
	
	string		bestWord, bestForm;
	long		bestScore;
};

inline
ostream & ComplexMaker::debug(ostream &os)
{
	os << "ComplexMaker: " << endl;
	os << "Best Word: " << bestWord << "  Best Form: " << 
			bestForm << "  Best Score: " << bestScore << endl;
	
	int i;
	for (i=0; i<words.size(); i++) os << words[i].Get() << ", "; os << endl;
	for (i=0; i<affix.size(); i++) affix[i].debug(os);
	os << endl;
	
	return os;
}

