/*
	Letter.cpp
	
	class Letter
	
	Encapsulates routines that deal with a single letter.
	We may want to make this table t
	
	James Jennings * April 18, 2000
*/

#include "Letter.h"
#include <string.h>
#include <ctype.h>

bool	Letter::IsVowel(char c, bool strict)
{	// Determine if a letter is a vowel.
	
	if (strict)	// doesn't include the borrowed 'w' and the schwa 'y'
		return strchr("aeiou", tolower(c)) != 0;
	else
		return strchr("aeiouwy", tolower(c)) != 0;
}

bool	Letter::IsConsonant(char c, bool strict)
{	// Determine if a letter is a consonant.
	
	if (strict)	// doesn't include borrowed letters
		return strchr("bcdfghjklmnprstvz", tolower(c)) != 0;
	else
		return strchr("bcdfghjklmnpqrstvxz", tolower(c)) != 0;
}

bool	Letter::IsBorrowed(char c)
{	// Determine if a letter is one of those used only for borrowed words.
	//	'q' = 'th' as in "thin"
	//	'x' = 'ch' as in "Bach"
	//	'w' = french 
	return strchr("qwx", tolower(c)) != 0;
}

bool 	Letter::IsContinuant(char c)
{	// Determine if a consonant is a continuant. Used in class Affix.
	// This table subject to revision.
	return strchr("cflmnrsvz", tolower(c)) != 0;
}

bool	Letter::IsWordLetter(char c)
{
	// A word can contain the letters A-Z, a-z, the apostrophe, and a non-terminal comma.
	// This routine can't check for the non-terminal comma.
	const char kCurlyQuote = 0xD5; // Mac Roman Right Single Quote
	char ch = tolower(c);
	return ( ('a' <= ch && ch <= 'z') || ch == '\'' || ch == kCurlyQuote || ch == '`' );
}

bool 	Letter::IsSpirant(char c)
{
	return strchr("cjsz", tolower(c)) != 0;
}

bool 	Letter::IsUnvoiced(char c)
{
	return strchr("ptkfsc", tolower(c)) != 0;
}

bool 	Letter::IsHyphen(char c)
{	// Determine if a letter could be a hyphen.
	return strchr("yrn", tolower(c)) != 0;
}
