/*
	Word.h
	
	class Word
	
	Encapsulates routines that deal with a Loglan word.
	
	James Jennings * April 18, 2000
*/

#pragma once

#include <string>
#include <ctype.h>

using std::string;

class Word {
public:
	typedef	string::size_type	size_type;
	
	// simple tests.
	static bool		IsName(const char *w);
	static bool		IsPredicate(const char *w);
	static bool		IsPrimitive(const char *w);
	static bool		IsLetter(const char *w);
	static bool		IsBorrowing(const char *w);
//	static bool		IsDigit(const char *w);
//	static bool		IsLW(const char *w);
	
	// utility
	static const char * PushUntilEndOfWord( string &dest, const char *w );
	
	// the encapsulated word
					Word( const char *w = 0);
	
	virtual const char *	Set(const char *w);
	virtual const string & 	Get() const;
	
	virtual bool			IsName() const { return IsName(Get().c_str()); }
	virtual bool			IsPredicate() const { return IsPredicate(Get().c_str()); }
	
protected:
	string	word;
};


/*
	StringCopyToLowerCase()
	Copy src to dest up to maxLength, converting to lower case.
	Always terminate the string.
	Return the destination string.
*/
inline
char * StringCopyToLowerCase( char *dest, const char *src, short maxLength)
{
	const char *p = src;
	char *q = dest;
	
	while ( *p != '\0' && (p-src) < maxLength-1 )
		*(q++) = tolower(*(p++));
	*q = '\0';
	
	return dest;
}
