// ===========================================================================
//	 CAffix.cp				1994-1998 Loglan Institute Inc. All rights reserved.
// ===========================================================================
// Routines specific to affixes

/*	JPJ: 6 Mar 99
	Neatened things up a little:
		Made functions static and const as appropriate
		Turned constants into enumerations.
		Turned all Booleans into bool (cross platform standard C++)
		Renamed things to make it clear that we work on 'normal' and 'short' affixes.
		
	Why does CalcAffixScore() have arrays for eight types, but there are only seven types?
	Make the CalcAffixScore() arrays static?
*/

#include "CAffix.h"
#include <string.h>
#include <assert.h>


// ===========================================================================
//	 Constructor		1994-1998 Loglan Institute Inc. All rights reserved.
// ===========================================================================
// 

CAffix::CAffix()
	: affType(CCV), affScore(0)
{
	Init();
	}
	
// ===========================================================================
//	 Destructor		1994-1998 Loglan Institute Inc. All rights reserved.
// ===========================================================================
// 

// CAffix::~CAffix() {}	// default destructor

// ===========================================================================
//	 Init				1994-1998 Loglan Institute Inc. All rights reserved.
// ===========================================================================
//

void	CAffix::Init()

{	
	aff[0] = '\0';  // Initialize the string
	form[0] = '\0';  // Initialize the string
	}
	
// ===========================================================================
//	 SetString			1994-1998 Loglan Institute Inc. All rights reserved.
// ===========================================================================
// 
void	CAffix::SetString(const char *theString)

{	assert(theString != 0 && strlen(theString) <= kMaxAffixSize );
	strcpy(aff,theString);
	}
	
// ===========================================================================
//	 SetAffixText		1994-1998 Loglan Institute Inc. All rights reserved.
// ===========================================================================
// 

CAffix::PrimType	CAffix::CalcAffix (bool isFinal)

{
	PrimType primType;
	
	affType = CCV;	// JPJ: this is what was in the original: is it the best default?
	switch (strlen(aff)) {
	case 1:
	case 2: strcat(aff,"r"); // Two letter affixes add 'r' if valid
	case 3:
		if (IsLettoral(aff)) affType = letr;
		else {
			affType = CVC;
// Previously this flagged an error. However adding 'y' to these has been discussed
			if (!IsLW(aff) ) aff[2] = 'y'; //was and may still be alert(1,155);
			}
		primType =  noType; // Nothing special to check for with this affix.
		break;
	case 5: // A standard primitive.  Affixes were inserted in CWordData
	
		primType = PType(aff);
		
		// a 5-letter predicate beginning with a vowel is a borrowing and adds 'y'
		
		if (!primType &&  !isFinal) strcat(aff,"y");
		else {
		
		// A 5-letter non-terminal affix replaces the final vowel with 'y'
		
			if( !isFinal ) aff[4] = 'y'; 
			}
		break;
	default:
		if ( !isFinal ) strcat(aff,"y");  // All others add 'y'
		break;
		}
	
	switch (primType) {
	case noType: 
		affScore = 100;
		break;
	case CCVCV:
		if ( ! isFinal ) affScore = 115;	// '!' added to match dictmake. 1/10/01
		else affScore = 0;
		break;
	default: // CVCCV
		if ( ! isFinal ) affScore = 65;	// '!' added to match dictmake. 1/10/01
		else affScore = 6;
		break;
		}
	
	if (form[0] == '\0' && strlen(aff) != 0) {
		// if the form hasn't been set yet, set it to the default
		strcpy(form, aff);
		if (affType != letr && !isFinal) {
			// parenthsize the final vowel
			int len = strlen(form);
			assert(len>0);
			char c = form[len-1];
			if (IsVowel(c) && len>0) {
				form[len-1] = '(';
				form[len]   = c;
				form[len+1] = ')';
				form[len+2] = '\0';
			}
		}
	}
	
	return primType; // a CWords data value
	}

// ===========================================================================
//	 CalcShortAffix		1994-1998 Loglan Institute Inc. All rights reserved.
// ===========================================================================
// Sets the type and score of an affix 

void CAffix::CalcShortAffix( bool isFinal)

{	
	if (aff[0]) {
		affType = Type(aff);
		affScore = CalcAffixScore(aff,affType,isFinal);
		}
		
	}

// ===========================================================================
//	 CalcAffixScore		1994-1998 Loglan Institute Inc. All rights reserved.
// ===========================================================================
// Determines an affix score dependent on its type

short CAffix::CalcAffixScore(char *affix, AffixType type, bool isFinal) 

{
	// the last values are those formed from the primitive (PTypes)
	// they are used in other routines
	const short final[8] = {132,0,154,154,176,176,0,6};
	const short nonFinal[8] = {363,371,373,373,390,390,115,65};
	const short conPlus = 20; // continuant CVC bonus (was 33 in an earlier draft)
	short score;
	
	if (isFinal) score = final[type];
	else score = nonFinal[type];
	if (type==CVC && IsContinuant(*(affix+2))) score += conPlus;
	return score;
	}

// ===========================================================================
//	 Type				1994-1998 Loglan Institute Inc. All rights reserved.
// ===========================================================================
// Determines the morphological form of an affix

CAffix::AffixType CAffix::Type(const char *aff)

{	if (IsCVC(aff)) return CVC;
	if (IsCCV(aff)) return CCV;
	if (IsCVVD(aff)) return (IsCVVO(aff))? diCVV:diCVv;
	if (IsCVV(aff)) return (IsCVVO(aff)) ? monoCVV:monoCVv;
	// JPJ: we don't expect to exit this way, but just in case...
	assert(false /*bad affix type*/);
	return CVC;
	}

// ===========================================================================
//	 PType				1994-1998 Loglan Institute Inc. All rights reserved.
// ===========================================================================
// Determines the morphology of a primitive

CAffix::PrimType CAffix::PType(const char *primitive) {

	if (IsVowel(*primitive)) return noType;
	else if (IsVowel(*(primitive+2))) return CCVCV; // 1
	else return CVCCV; // 2
	}

// === static methods ===

// ===========================================================================
//	 IsVowel			1994-1998 Loglan Institute Inc. All rights reserved.
// ===========================================================================
// Determine if a letter is a vowel.

bool CAffix::IsVowel(char c) 

{
	return (c=='a' || c=='e' || c== 'i' || c== 'o' || c== 'u');
	}

// ===========================================================================
//	 IsLettoral		1994-1998 Loglan Institute Inc. All rights reserved.
// ===========================================================================
// Determine if a consonant is a continuant. This table subject to revision.

bool CAffix::IsContinuant(char c)

{	return (c=='c' || c=='f' || c=='l' || c=='m' || c=='n'
		|| c=='r' || c=='s' || c=='v'|| c=='z');
	}
	
	
// ===========================================================================
//	 IsLW				1994-1998 Loglan Institute Inc. All rights reserved.
// ===========================================================================
// Checks for one of the little words that can be combined into complexes.

bool CAffix::IsLW(const char *theString) 

{	char i;
	
	const char 	*lwSet[] = {
					"fur","jur","nor","nur","rar","ror","nir","ner","tor","ter",
					"for","fer","por","sor","ser","vor","ver","rer","pir","sur",
					"nun","fun", 
					0};	// JPJ: zero marks the end of the array

	for (i=0; lwSet[i] != 0; ++i) {
		if (!strcmp (theString, lwSet[i])) return true;
		}
	return false;
	}
	
// ===========================================================================
//	 IsLettoral		1994-1998 Loglan Institute Inc. All rights reserved.
// ===========================================================================
// Checks if an affix is a letter word

bool CAffix::IsLettoral(const char *theString) 

{
	return (strstr(theString,"ai") || strstr(theString,"ei") || strstr(theString,"eo"));
	}

// ===========================================================================
//	 IsCVC			1994-1998 Loglan Institute Inc. All rights reserved.
//	 IsCVV
//   IsCVVD
//   IsCVVO
//   IsCCV
// ===========================================================================
// Straightforward morphology checks on affixes

bool CAffix::IsCVC(const char *aff)  

{
	return (strlen(aff)==3 && IsVowel (*(aff+1)) && !IsVowel(*(aff+2)));
	}

	
bool CAffix::IsCVV(const char *c)

/*Determine if an affix consists of a consonant followed by two vowels*/

{	return (IsVowel(*(c+1))&&IsVowel (*(c+2)));
	}
	
bool CAffix::IsCVVD(const char *a)

/*Determine if a CVV type affix is pronounced disyllabically (TRUE) or monosyllabically (FALSE) */

{	if(!IsCVV(a)) return (false);
	if (a[1]=='i'&&a[2]!='i') return (false);
	if (a[1]=='u'&&a[2]!='u') return (false);
	if (a[2]=='i'&&a[1]!='i') return (false);
	if (a[1]=='a'&&a[2]=='o') return (false);
	return (true);
	}

bool CAffix::IsCVVO(const char *c)

/*Determine if a CVV affix is terminated by two identical vowels*/

{	return (*(c+1) == *(c+2)); /*An affix cannot terminate with 2 cons.*/
	}


bool CAffix::IsCCV(const char *c)

//Determine if an affix consists of two consonants followed by a vowel

{	return ((strlen(c) == 3) && !IsVowel (*(c+1)) && IsVowel (*(c+2)));
	}
	
