// ===========================================================================
//	 CAffix.h				1994-1998 Loglan Institute Inc. All rights reserved.
// ===========================================================================
// Handle routines specific to Affixes

#ifndef _Aff
#define _Aff
#pragma once

//#include "CPrim.h"
#include <string.h>
#include <assert.h>

class CAffix {
	enum { kMaxAffixSize = 20/*14*/, kMaxFormSize = kMaxAffixSize + 2 };
	public:
		// JPJ: enumerations for Primitive and Affix types
		typedef enum {	noType,
						CCVCV, 
						CVCCV } PrimType;
		typedef enum {	CCV = 0,
						CVC = 1,
						diCVv = 2,
						diCVV = 3,
						monoCVv = 4,
						monoCVV = 5,
						letr = 6 } AffixType;
		
						CAffix();
	//	virtual			~CAffix();
		
		// calculation methods
		void			CalcShortAffix(bool isFinal); //Set type and score of an affix
		PrimType		CalcAffix(bool isFinal); //Acquire or form the affix
		
	public:
		// Accessors
		void 			SetString(const char *theString); // Set an affix value to theString
		AffixType		GetAffixType() const;
		short			GetAffixScore() const;
		const char 	*	GetAffixText() const;
		
		void			SetForm( const char inForm[] );
		const char	*	GetForm() const;
		
		// A series of simple tests of affix structure.
		// (JPJ: 'static' because they can be called without an object)
		
		static bool		IsVowel(char c);
		static bool 	IsLW(const char *theString); 
		static bool		IsLettoral(const char *theString);
		static bool		IsCVC(const char *aff);
		static bool		IsCCV(const char *aff);
		static bool		IsCVV(const char *aff);
		static bool		IsCVVD(const char *aff);
		static bool		IsCVVO(const char *aff);
		static bool 	IsContinuant(char c);
	
	protected:
		short 			CalcAffixScore(char *affix, AffixType type, bool isFinal);
		PrimType		PType(const char *primitive); // Determine the primitive type
		AffixType		Type(const char *aff); // Determine the type of an affix
	
		AffixType	affType;	// This is the morphology e.g. CCV, CVC etc.
		short 		affScore;	// This is a score dependent on position and morphology
		char 		aff[kMaxAffixSize+1];	// The affix text itself
		char		form[kMaxFormSize+1];

	private:
		// constructor shared initialization code should usually be private
		void 			Init();						// Initialization
 	
 	};
 	
 // Inline Accessors
 
inline
CAffix::AffixType	CAffix::GetAffixType() const { return affType; }

inline
const char *		CAffix::GetAffixText() const { return aff; }

inline
short				CAffix::GetAffixScore() const { return affScore; }

inline
void				CAffix::SetForm( const char inForm[] )
{
	assert(strlen(inForm) <= kMaxFormSize);
	strcpy(form, inForm);
}

inline
const char	*		CAffix::GetForm() const { return form; }

 #endif
