#ifndef _WdDat
#define _WdDat
#pragma once

// ===========================================================================
//	 CWordMaker.h 			1994-1998 Metrowerks Inc. All rights reserved.
// ===========================================================================
//	This class orchestrates the formation of canonical words from lists of primitives

#include "CAffix.h"
#include "CWord.h"
//#include "CPrim.h"
//#include "LODConstants.h"
//#include "CFiles.h"

class CWordMaker {
	enum { kWordLength = 25, kMaxWords = 5 };
	public:
						CWordMaker();		// default constructor
						CWordMaker(const char *wordList);
						CWordMaker(const unsigned char *wordList);
	//	virtual			~CWordMaker();
		
		void			SetWords(const char *wordList);	// set the pieces to build from
		bool			CanGetBest() const;
		const char *	GetBestWord() const;		// Accessor for the best complex
		const char *	GetBestWordForm() const;
		
		static const char *	UnmangleForm( const char inForm[], char outString[]);
		static int		GetComplexity( const char inForm[] );
		virtual int		GetComplexity();
		
	protected:
		void 			ParseWordList(char *theString);	// Recover the primitives
		void 			CalcAffixes();				// Get the affixes for each primitive
			
		char *			FindBest();					// Find the best affix combination
		short			GetPrimCount() const;		// Accessor for the primCount
		CWord			GetWordText(short i);	// Accessor for the word class
		void 			MakeWord(short *theAffix);	// Test each affix combination
	
	public:
/* This series of boolean tests determine whether a primitive has been borrowed, whether
a certain triple consonant string is valid, whether a junction between two affixes is 
an allowed one, whether a consonant pair is a potential initial consonant pair, and 
whether certain consonants are sprirants or unvoiced */
		
		// ('static' because they can be called without a CWordMaker object)
		static bool 	IsBorrowed(const char *theString);
		static bool 	IsBadCccJoint(char a, char b, char c);
		static bool 	IsValidJunction(char a, char b);
		static bool  	IsInitialPair(char a, char b);
		static bool 	IsSpirant(char a);
		static bool 	IsUnvoiced(char a);
		static bool		IsVowel(char a) { return CAffix::IsVowel(a); }

	protected:
		
		char 	testWord[kWordLength+1];// Holds the current word being tested
		char 	bestWord[kWordLength+1];// Holds the current best candidate word
		char 	testForm[2*kWordLength+1];// Holds the current word form
		char 	bestForm[2*kWordLength+1];// Holds the best candidate word's form
		
	//	short	affCount;			// the number of affixes for a primitive
		short 	testScore;			// the score for the current complex
		short 	bestScore;			// the score for the current best complex
		short 	primCount;			// the number of primitives being combined
		
		CWord	wd[kMaxWords];		// a substructure of data about each primitive
	
	private:
		// constructor shared initialization code should usually be private
		void 			Init();				// Initialization
		
	};

// Accessors

inline
bool CWordMaker::CanGetBest() const { return CPrimSet::Good() && primCount > 1; }

inline 
const char * CWordMaker::GetBestWord() const { return bestWord; }

inline 
const char * CWordMaker::GetBestWordForm() const { return bestForm; }

inline 
short CWordMaker::GetPrimCount() const { return primCount; }

inline // JPJ: Not a pointer? Should return a reference?
CWord  CWordMaker::GetWordText(short i)	{ return wd[i]; }

#endif
